/*******************************************************************************

       MICTRIO ELETRNICO

       c/ PIC 12F675

       Autor: Cludio Lrios

       Data: 24/10/2016

       Aplicao: Ativar descarga de forma automtica em mictrios, com sensor
       de presena infrared.
       
       Pinagem:
       1= vcc
       2= nc
       3= nc
       4= sada do receptor tsop1838
       5- sada para acionamento do rele e eletrovlvula de gua
       6- led indicador de acionamento
       7- sada para o led infravermelho
       8- gnd

       Uso didtico

       Este arquivo  parte integrante do blog Lrios.Tecnologia.WS


******************************************************************************/
#include <12F675.h>
#device adc=8
#use delay(clock=4000000)
#fuses NOWDT,INTRC_IO, NOCPD, NOPROTECT, NOMCLR, NOPUT, NOBROWNOUT//configurao
//Certifique que o byte de calibrao do seu pic no tenha sido perdido, sob 
//pena de no funcionar.

//bytes
#byte TMR0   = 0x01
#byte TRISIO = 0x85
#byte GPIO   = 0x05
#byte OPTION_REG = 0x81
#byte ANSEL  = 0x9f
#byte CMCON  = 0x19
#byte ADCON  = 0X1F
#byte ADRESH = 0X1E
#byte ADRESL = 0X9E


//bits
#bit  IN  = 0x5.3  //entrada do fotoreceptor - pino 4
#bit  OUT = 0x5.2 //sada para controlar eletrovlvula - pino 5
#bit  LED = 0x5.1 //led indicador de programao
#bit  LEDIR = 0x5.0 //led emissor de infrared

#bit  t0if = 0xb.2 //flag de over flow do timer0
#bit  godone = 0x1f.1
#bit  adfm   = 0x1f.7
#bit  adon   = 0x1f.0
#bit  vcfg   = 0x1f.6
#bit  f_int0 = 0xb.5

//Defines
#define TEMPO_DESCARGA 5000 //5 seg - tempo que ativa a eletrovlvula 
#define FILTRO1  49
#define TEMPO_NF 500  //5 segundos na frente do sensor
#define TEMPO_POS_DESCARGA 5000// 5 segundos
#define FREQ_PISCADA 100  //100 milisegundos

//variveis globais
unsigned int16 cont1;
/*******************************************************************************
                 Rotina do Sensor de Aproximao
*******************************************************************************/

unsigned int1 sensor_ir(){
//Para TSOP1838 ou similar (KSM603LM,VS1838) de frequncia = 38khz

     unsigned int8 pulso,b;
     pulso=0;

     for(b=0;b<80;b++){ //gera 80 pulsos na frequncia prxima de 38 khz
       LEDIR=0;
       delay_us(12);
       LEDIR=1;
       if(!IN)pulso++; else delay_cycles( 2 );//testa se pulso chegou no receptor
     }
      LEDIR=0;
      delay_ms(15);
     if(pulso > FILTRO1)  return (1); else return (0);

}

/*******************************************************************************
                 ROTINA MAIN
*******************************************************************************/

void main() {

   setup_adc_ports(NO_ANALOGS|VSS_VDD);//ports digitais
   setup_adc(ADC_OFF);//desliga AD
   setup_counters(RTCC_INTERNAL,RTCC_DIV_1);//n usado
   setup_timer_1(T1_DISABLED);//n usado
   setup_comparator(NC_NC_NC_NC);//desliga comparadores internos
   setup_vref(FALSE);//desliga voltagem de referncia
   TRISIO=0B111000;//ajusta sadas e entradas
   cont1=0;//zera contador
   GPIO=0;//desliga todas as sadas
/*******************************************************************************
                 LAO PRINCIPAL DE REPETIO
*******************************************************************************/

   for(;;){

     if(sensor_ir()){//se pessoa est na frente do sensor ento incrementa 
        cont1++;
     }
     else{
        if(cont1<TEMPO_NF)cont1=0;//zera se sair da frente do sensor
                                  //enquanto o tempo for menor
     }

     if(cont1>=TEMPO_NF){//pessoa ficou mais de 10 segundos na frente do sensor

        while(sensor_ir()){//espera pessoa sair da frente do sensor
         LED=!LED;//pisca o led p/ avisar que ir dar descarga ao sair da frente
         delay_ms(FREQ_PISCADA);//determina a velocidade da piscada
        }
        OUT=1;//aciona sada da eletrovalvula
        LED=1;//liga led
        delay_ms(TEMPO_DESCARGA);//tempo que fica sando gua
        LED=0;//apaga led
        OUT=0;//desliga eletrovalvula
        cont1=0;//zera contador de tempo
        delay_ms(TEMPO_POS_DESCARGA);//tempo se acionamento
     }//if(cont1>=TEMPO_NF)
  }//for()
}//main

